JSON_BODY_PARSE_NAME = "body"
REQUEST_INPUT_SCHEMA_NAME = "input"
REQUEST_INPUT_DATA_SCHEMA_NAME = "input_data"
REQUEST_INPUT_FORM_SCHEMA_NAME = "input_form"
REQUEST_INPUT_QUERY_SCHEMA_NAME = "input_query"
RESPONSE_OUTPUT_SCHEMA_NAME = "output"
JSON_OUTPUT_PARSE_NAME = "response"
